<?php

namespace Drupal\supercache\Tests\Cache;

use Drupal\supercache\Tests\Generic\Cache\CacheTagsChecksumTests as TagsChecksumTests;

use Drupal\couchbasedrupal\Cache\CouchbaseRawBackendFactory;
use Drupal\couchbasedrupal\CouchbaseManager;
use Drupal\wincachedrupal\Cache\WincacheRawBackendFactory;

use Drupal\supercache\Cache\ApcuRawBackendFactory;
use Drupal\supercache\Cache\DatabaseRawBackendFactory;
use Drupal\supercache\Cache\CacheCacheTagsChecksum;
use Drupal\supercache\Cache\ChainedFastRawBackendFactory;

use Drupal\Core\Site\Settings;
use Drupal\Core\Cache\DatabaseBackendFactory;

/**
 * Test this on top of different combinations of cache backends.
 */
class CacheCacheTagsChecksumTests extends TagsChecksumTests {

  /**
   * List of cache backends to perform the tests on.
   *
   * @var \Drupal\supercache\Cache\CacheRawFactoryInterface[]
   */
  protected $backends;

  public function setUp() {

    parent::setUp();

    /** @var \Psr\Log\LoggerInterface */
    $logger =  $this->getMock(\Psr\Log\LoggerInterface::class);
    $app_root = '/';
    $site_path = 'mypath';

    // The testing system chokes
    // when database used during shutdown.
    new Settings(
      ['fastchained_disable_shutdown' => TRUE]
      );

    $settings = \Drupal\Core\Site\Settings::getInstance();
    $manager = new CouchbaseManager($settings, $logger);
    $connection = \Drupal\Core\Database\Database::getConnection();

    // TODO: Split each combination into it's own test and
    // move to the proper module.

    $this->container->set('rawbackend.wincache', new WincacheRawBackendFactory($app_root, $site_path));
    $this->container->set('rawbackend.couchbase', new CouchbaseRawBackendFactory($manager, $app_root, $site_path));
    $this->container->set('rawbackend.apcu', new ApcuRawBackendFactory($app_root, $site_path));
    $this->container->set('rawbackend.database', new DatabaseRawBackendFactory($connection));

    $this->backends[] = $this->container->get('rawbackend.wincache');
    $this->backends[] = $this->container->get('rawbackend.couchbase');
    $this->backends[] = $this->container->get('rawbackend.apcu');
    $this->backends[] = $this->container->get('rawbackend.database');


    // Now let's have fun with ChainedRaw

    // Wincache + Couchbase
    $f = new ChainedFastRawBackendFactory($settings, 'rawbackend.couchbase', 'rawbackend.wincache');
    $f->setContainer($this->container);
    $this->backends[] = $f;
    // Wincache + Database
    $f = new ChainedFastRawBackendFactory($settings, 'rawbackend.database', 'rawbackend.wincache');
    $f->setContainer($this->container);
    $this->backends[] = $f;
    // Apcu + Couchbase
    $f = new ChainedFastRawBackendFactory($settings, 'rawbackend.couchbase', 'rawbackend.apcu');
    $f->setContainer($this->container);
    $this->backends[] = $f;
    // Apcu + Database
    $f = new ChainedFastRawBackendFactory($settings, 'rawbackend.database', 'rawbackend.apcu');
    $f->setContainer($this->container);
    $this->backends[] = $f;
    // Couchbase + Database
    $f = new ChainedFastRawBackendFactory($settings, 'rawbackend.database', 'rawbackend.couchbase');
    $f->setContainer($this->container);
    $this->backends[] = $f;
  }

  public function testTagInvalidations() {
    foreach ($this->backends as $backend) {
      // Invalidator and provider on top of the cache.
      $provider = new CacheCacheTagsChecksum($backend);
      $this->tagInvalidator = $provider;
      $this->tagChecksum = $provider;
      // The backend factory will be the same for all
      $connection = \Drupal\Core\Database\Database::getConnection();
      $this->backendFactory = new DatabaseBackendFactory($connection, $this->tagChecksum);
      // Call parent tests.
      parent::testTagInvalidations();
    }
  }

  public function tearDown() {

  }
}
